#!/bin/env python

"""Module providing classes for rendering HTML month, week and other
calendars from HTML templates.

Uses evcal.base.EventTable and upytestry.Renderer classes."""

import upytestry
import calendar
from base import EventTable
from datetime import datetime, timedelta

class Cal:
    "Dummy class, to be removed."
    def __init__(self):
        self.weekdays = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]

class CalendarRenderer(upytestry.Renderer):
    "Base class for all calendar renderers."
    
    def __init__(self, template, cal, params):
        upytestry.Renderer.__init__(self, template)
        self.cal = cal
        self.params = params

class MonthCalendarRenderer(CalendarRenderer):

    class MonthEventTable(EventTable):
        def __init__(self, events, start, end):
            EventTable.__init__(self, events, start, end, timedelta(days=1))

        def extractColIndex(self, d):
            return calendar.weekday(d.year, d.month, d.day)

        def isNewRow(self, end):
            return calendar.weekday(end.year, end.month, end.day)==0
    
    def __init__(self, events, year, month, cal, params):
        if not hasattr(MonthCalendarRenderer, "monthTemplate"):
            p = upytestry.Parser()
            p.feed(open("monthly.html.tmpl").read())
            MonthCalendarRenderer.monthTemplate = p.html
        
        CalendarRenderer.__init__(self, MonthCalendarRenderer.monthTemplate, cal, params)

        self.year = year
        self.month = month
        
        start = datetime(year=year, month=month, day=1)
        wd = calendar.weekday(year, month, 1)
        start = start - timedelta(days=wd)
        self.start = start

        end = datetime(year=year, month=month+1, day=1)
        wd = calendar.weekday(year, month+1, 1)
        end = end + timedelta(days=7-wd)
        self.end = end
        
        self.table = MonthCalendarRenderer.MonthEventTable(events, self.start, self.end)

        self.date = self.start
        self.step = timedelta(days=1)
        self.buffers = upytestry.Buffers(self)

    def currentMonth(self):
        return self.date.month==self.month

class WeekCalendarRenderer(CalendarRenderer):

    class WeekEventTable(EventTable):
        def __init__(self, events, start, end):
            EventTable.__init__(self, events, start, end, timedelta(hours=1), hasColGroup=True)

        def extractColIndex(self, d):
            return d.hour

        def isNewRow(self, end):
            return end.hour==0

        def isColGroupEvent(self, ev):
            return ev.start.day != ev.end.day
    
    def __init__(self, events, date, cal, params):
        if not hasattr(WeekCalendarRenderer, "weekTemplate"):
            p = upytestry.Parser()
            p.feed(open("weekly.html.tmpl").read())
            WeekCalendarRenderer.weekTemplate = p.html
        
        CalendarRenderer.__init__(self, WeekCalendarRenderer.weekTemplate, cal, params)

        start = datetime(year=date.year, month=date.month, day=date.day)
        wd = calendar.weekday(date.year, date.month, date.day)
        start = start - timedelta(days=wd)
        self.start = start

        end = start + timedelta(days=7)
        self.end = end
        
        self.table = WeekCalendarRenderer.WeekEventTable(events, self.start, self.end)

        self.date = self.start
        self.step = timedelta(hours=1)
        self.buffers = upytestry.Buffers(self)

    def getColGroupEventSpan(self, colev):
        ev, from_, to = colev
        span = 0
        for i in xrange(from_, to):
            span = span + max(1, len(self.table.groups[i]))
        return span

